
/* This file is generated by glib-mkenums, do not modify it. This code is licensed under the same license as the containing project. Note that it links to GLib, so must comply with the LGPL linking clauses. */

#include "enum-types.h"
#include "chatty-enums.h"

#define C_ENUM(v) ((gint) v)
#define C_FLAGS(v) ((guint) v)

/* enumerations from "chatty-enums.h" */

GType
chatty_status_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (CHATTY_UNKNOWN), "CHATTY_UNKNOWN", "unknown" },
        { C_ENUM (CHATTY_DISCONNECTED), "CHATTY_DISCONNECTED", "disconnected" },
        { C_ENUM (CHATTY_CONNECTING), "CHATTY_CONNECTING", "connecting" },
        { C_ENUM (CHATTY_CONNECTED), "CHATTY_CONNECTED", "connected" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("ChattyStatus"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
chatty_id_type_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (CHATTY_ID_UNKNOWN), "CHATTY_ID_UNKNOWN", "unknown" },
        { C_ENUM (CHATTY_ID_PHONE), "CHATTY_ID_PHONE", "phone" },
        { C_ENUM (CHATTY_ID_EMAIL), "CHATTY_ID_EMAIL", "email" },
        { C_ENUM (CHATTY_ID_MATRIX), "CHATTY_ID_MATRIX", "matrix" },
        { C_ENUM (CHATTY_ID_XMPP), "CHATTY_ID_XMPP", "xmpp" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("ChattyIdType"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
chatty_protocol_get_type (void)
{
    static gsize gtype_id = 0;
    static const GFlagsValue values[] = {
        { C_FLAGS (CHATTY_PROTOCOL_NONE), "CHATTY_PROTOCOL_NONE", "none" },
        { C_FLAGS (CHATTY_PROTOCOL_MMS_SMS), "CHATTY_PROTOCOL_MMS_SMS", "mms-sms" },
        { C_FLAGS (CHATTY_PROTOCOL_MMS), "CHATTY_PROTOCOL_MMS", "mms" },
        { C_FLAGS (CHATTY_PROTOCOL_CALL), "CHATTY_PROTOCOL_CALL", "call" },
        { C_FLAGS (CHATTY_PROTOCOL_XMPP), "CHATTY_PROTOCOL_XMPP", "xmpp" },
        { C_FLAGS (CHATTY_PROTOCOL_MATRIX), "CHATTY_PROTOCOL_MATRIX", "matrix" },
        { C_FLAGS (CHATTY_PROTOCOL_TELEGRAM), "CHATTY_PROTOCOL_TELEGRAM", "telegram" },
        { C_FLAGS (CHATTY_PROTOCOL_EMAIL), "CHATTY_PROTOCOL_EMAIL", "email" },
        { C_FLAGS (CHATTY_PROTOCOL_DELTA), "CHATTY_PROTOCOL_DELTA", "delta" },
        { C_FLAGS (CHATTY_PROTOCOL_THREEPL), "CHATTY_PROTOCOL_THREEPL", "threepl" },
        { C_FLAGS (CHATTY_PROTOCOL_LAST), "CHATTY_PROTOCOL_LAST", "last" },
        { C_FLAGS (CHATTY_PROTOCOL_ANY), "CHATTY_PROTOCOL_ANY", "any" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_flags_register_static (g_intern_static_string ("ChattyProtocol"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
chatty_encryption_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (CHATTY_ENCRYPTION_UNKNOWN), "CHATTY_ENCRYPTION_UNKNOWN", "unknown" },
        { C_ENUM (CHATTY_ENCRYPTION_ENABLED), "CHATTY_ENCRYPTION_ENABLED", "enabled" },
        { C_ENUM (CHATTY_ENCRYPTION_DISABLED), "CHATTY_ENCRYPTION_DISABLED", "disabled" },
        { C_ENUM (CHATTY_ENCRYPTION_UNSUPPORTED), "CHATTY_ENCRYPTION_UNSUPPORTED", "unsupported" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("ChattyEncryption"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
chatty_user_flag_get_type (void)
{
    static gsize gtype_id = 0;
    static const GFlagsValue values[] = {
        { C_FLAGS (CHATTY_USER_FLAG_NONE), "CHATTY_USER_FLAG_NONE", "none" },
        { C_FLAGS (CHATTY_USER_FLAG_MEMBER), "CHATTY_USER_FLAG_MEMBER", "member" },
        { C_FLAGS (CHATTY_USER_FLAG_MODERATOR), "CHATTY_USER_FLAG_MODERATOR", "moderator" },
        { C_FLAGS (CHATTY_USER_FLAG_OWNER), "CHATTY_USER_FLAG_OWNER", "owner" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_flags_register_static (g_intern_static_string ("ChattyUserFlag"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
chatty_msg_direction_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (CHATTY_DIRECTION_UNKNOWN), "CHATTY_DIRECTION_UNKNOWN", "unknown" },
        { C_ENUM (CHATTY_DIRECTION_IN), "CHATTY_DIRECTION_IN", "in" },
        { C_ENUM (CHATTY_DIRECTION_OUT), "CHATTY_DIRECTION_OUT", "out" },
        { C_ENUM (CHATTY_DIRECTION_SYSTEM), "CHATTY_DIRECTION_SYSTEM", "system" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("ChattyMsgDirection"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
chatty_msg_status_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (CHATTY_STATUS_UNKNOWN), "CHATTY_STATUS_UNKNOWN", "unknown" },
        { C_ENUM (CHATTY_STATUS_DRAFT), "CHATTY_STATUS_DRAFT", "draft" },
        { C_ENUM (CHATTY_STATUS_RECEIVED), "CHATTY_STATUS_RECEIVED", "received" },
        { C_ENUM (CHATTY_STATUS_SENDING), "CHATTY_STATUS_SENDING", "sending" },
        { C_ENUM (CHATTY_STATUS_SENT), "CHATTY_STATUS_SENT", "sent" },
        { C_ENUM (CHATTY_STATUS_DELIVERED), "CHATTY_STATUS_DELIVERED", "delivered" },
        { C_ENUM (CHATTY_STATUS_READ), "CHATTY_STATUS_READ", "read" },
        { C_ENUM (CHATTY_STATUS_SENDING_FAILED), "CHATTY_STATUS_SENDING_FAILED", "sending-failed" },
        { C_ENUM (CHATTY_STATUS_DELIVERY_FAILED), "CHATTY_STATUS_DELIVERY_FAILED", "delivery-failed" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("ChattyMsgStatus"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
chatty_msg_type_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (CHATTY_MESSAGE_UNKNOWN), "CHATTY_MESSAGE_UNKNOWN", "unknown" },
        { C_ENUM (CHATTY_MESSAGE_TEXT), "CHATTY_MESSAGE_TEXT", "text" },
        { C_ENUM (CHATTY_MESSAGE_HTML), "CHATTY_MESSAGE_HTML", "html" },
        { C_ENUM (CHATTY_MESSAGE_HTML_ESCAPED), "CHATTY_MESSAGE_HTML_ESCAPED", "html-escaped" },
        { C_ENUM (CHATTY_MESSAGE_MATRIX_HTML), "CHATTY_MESSAGE_MATRIX_HTML", "matrix-html" },
        { C_ENUM (CHATTY_MESSAGE_LOCATION), "CHATTY_MESSAGE_LOCATION", "location" },
        { C_ENUM (CHATTY_MESSAGE_FILE), "CHATTY_MESSAGE_FILE", "file" },
        { C_ENUM (CHATTY_MESSAGE_IMAGE), "CHATTY_MESSAGE_IMAGE", "image" },
        { C_ENUM (CHATTY_MESSAGE_VIDEO), "CHATTY_MESSAGE_VIDEO", "video" },
        { C_ENUM (CHATTY_MESSAGE_AUDIO), "CHATTY_MESSAGE_AUDIO", "audio" },
        { C_ENUM (CHATTY_MESSAGE_MMS), "CHATTY_MESSAGE_MMS", "mms" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("ChattyMsgType"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
chatty_file_status_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (CHATTY_FILE_UNKNOWN), "CHATTY_FILE_UNKNOWN", "unknown" },
        { C_ENUM (CHATTY_FILE_DOWNLOADED), "CHATTY_FILE_DOWNLOADED", "downloaded" },
        { C_ENUM (CHATTY_FILE_MISSING), "CHATTY_FILE_MISSING", "missing" },
        { C_ENUM (CHATTY_FILE_DECRYPT_FAILED), "CHATTY_FILE_DECRYPT_FAILED", "decrypt-failed" },
        { C_ENUM (CHATTY_FILE_DOWNLOADING), "CHATTY_FILE_DOWNLOADING", "downloading" },
        { C_ENUM (CHATTY_FILE_ERROR), "CHATTY_FILE_ERROR", "error" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("ChattyFileStatus"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
chatty_item_state_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (CHATTY_ITEM_VISIBLE), "CHATTY_ITEM_VISIBLE", "visible" },
        { C_ENUM (CHATTY_ITEM_HIDDEN), "CHATTY_ITEM_HIDDEN", "hidden" },
        { C_ENUM (CHATTY_ITEM_ARCHIVED), "CHATTY_ITEM_ARCHIVED", "archived" },
        { C_ENUM (CHATTY_ITEM_BLOCKED), "CHATTY_ITEM_BLOCKED", "blocked" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("ChattyItemState"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
chatty_chat_state_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (CHATTY_CHAT_UNKNOWN), "CHATTY_CHAT_UNKNOWN", "unknown" },
        { C_ENUM (CHATTY_CHAT_VERIFICATION), "CHATTY_CHAT_VERIFICATION", "verification" },
        { C_ENUM (CHATTY_CHAT_INVITED), "CHATTY_CHAT_INVITED", "invited" },
        { C_ENUM (CHATTY_CHAT_JOINED), "CHATTY_CHAT_JOINED", "joined" },
        { C_ENUM (CHATTY_CHAT_LEFT), "CHATTY_CHAT_LEFT", "left" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("ChattyChatState"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* Generated data ends here */

